//开源许可证声明
/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
//声明了当前Java文件所属的包名。包名用于组织和管理Java类，避免命名冲突。
package net.micode.notes.gtask.data;
//
import android.content.ContentResolver;//用于与内容提供者进行交互，执行CRUD（创建、读取、更新、删除）操作。
import android.content.ContentUris;//用于构建URI，支持通过URI访问和操作数据。
import android.content.ContentValues;//用于存储和操作键值对数据。
import android.content.Context;//提供应用程序上下文信息，用于访问系统服务和资源。
import android.database.Cursor;//用于遍历和操作数据库中的数据行。
import android.net.Uri;//用于表示URI，支持通过URI访问和操作数据。
import android.util.Log;//用于输出日志信息，方便调试和跟踪。

import net.micode.notes.data.Notes;//表示小米便签应用中的Notes类，用于访问和操作便签数据。
import net.micode.notes.data.Notes.DataColumns;//定义了Notes类中的数据列，用于访问和操作便签数据。
import net.micode.notes.data.Notes.DataConstants;//定义了Notes类中的常量，用于访问和操作便签数据。
import net.micode.notes.data.Notes.NoteColumns;//定义了Notes类中的数据列，用于访问和操作便签数据。
import net.micode.notes.data.NotesDatabaseHelper.TABLE;//表示小米便签应用中的NotesDatabaseHelper类，用于访问和操作便签数据。
import net.micode.notes.gtask.exception.ActionFailureException;//表示小米便签应用中的ActionFailureException类，用于处理操作失败的情况。

import org.json.JSONException;//用于处理JSON解析和操作中的异常。
import org.json.JSONObject;//用于表示JSON对象，支持通过键值对存储和操作数据。


public class SqlData {
    // 定义一个常量TAG，用于标识类的简单名称
    private static final String TAG = SqlData.class.getSimpleName();

    // 定义一个常量INVALID_ID，用于标识无效的ID
    private static final int INVALID_ID = -99999;

    // 定义一个常量PROJECTION_DATA，用于存储数据的列名
    public static final String[] PROJECTION_DATA = new String[] {//列名常量列表
            DataColumns.ID, DataColumns.MIME_TYPE, DataColumns.CONTENT, DataColumns.DATA1,
            DataColumns.DATA3
    };

    // 定义一个常量DATA_ID_COLUMN，用于标识数据的ID列
    public static final int DATA_ID_COLUMN = 0;

    // 定义一个常量DATA_MIME_TYPE_COLUMN，用于标识数据的MIME类型列
    public static final int DATA_MIME_TYPE_COLUMN = 1;

    // 定义一个常量DATA_CONTENT_COLUMN，用于标识数据的CONTENT列
    public static final int DATA_CONTENT_COLUMN = 2;

    // 定义一个常量DATA_CONTENT_DATA_1_COLUMN，用于标识数据的DATA1列
    public static final int DATA_CONTENT_DATA_1_COLUMN = 3;

    // 定义一个常量DATA_CONTENT_DATA_3_COLUMN，用于标识数据的DATA3列
    public static final int DATA_CONTENT_DATA_3_COLUMN = 4;

    // 定义一个ContentResolver对象mContentResolver，用于访问内容提供者
    private ContentResolver mContentResolver;

    // 定义一个布尔值mIsCreate，用于标识是否创建
    private boolean mIsCreate;

    // 定义一个长整型变量mDataId，用于存储数据的ID
    private long mDataId;

    // 定义一个字符串变量mDataMimeType，用于存储数据的MIME类型
    private String mDataMimeType;

    // 定义一个字符串变量mDataContent，用于存储数据的CONTENT
    private String mDataContent;

    // 定义一个长整型变量mDataContentData1，用于存储数据的DATA1
    private long mDataContentData1;

    // 定义一个字符串变量mDataContentData3，用于存储数据的DATA3
    private String mDataContentData3;

    // 定义一个ContentValues对象mDiffDataValues，用于存储数据的差异
    private ContentValues mDiffDataValues;


    // 构造函数，创建并初始化SqlData对象
    public SqlData(Context context) {
        // 获取ContentResolver对象，ContentResolver用于访问Android系统中的内容提供者
        mContentResolver = context.getContentResolver();
        // 设置mIsCreate为true，表示正在创建新的数据
        mIsCreate = true;
        // 设置mDataId为INVALID_ID，表示数据ID无效
        mDataId = INVALID_ID;
        // 设置mDataMimeType为DataConstants.NOTE，表示数据类型为Note
        mDataMimeType = DataConstants.NOTE;
        // 设置mDataContent为空字符串，可能用于存储数据内容。
        mDataContent = "";
        // 设置mDataContentData1为0，可能用于存储数据的其他信息。
        mDataContentData1 = 0;
        // 设置mDataContentData3为空字符串，可能用于存储数据的其他信息。
        mDataContentData3 = "";
        // 初始化mDiffDataValues为ContentValues对象，用于存储数据的变化。
        mDiffDataValues = new ContentValues();
    }

    // 构造函数，用于从Cursor中加载数据
    public SqlData(Context context, Cursor c) {
        // 获取ContentResolver对象，ContentResolver用于访问Android系统中的内容提供者
        mContentResolver = context.getContentResolver();
        // 设置mIsCreate为false，表示数据已经存在
        mIsCreate = false;
        // 从Cursor中加载数据
        loadFromCursor(c);
        // 初始化mDiffDataValues为ContentValues对象，用于存储数据的变化。
        mDiffDataValues = new ContentValues();
    }

    // 从Cursor中加载数据
    private void loadFromCursor(Cursor c) {
        // 从 Cursor 中获取数据的 ID 列的值，并赋值给成员变量 mDataId
        mDataId = c.getLong(DATA_ID_COLUMN);
        // 获取数据类型 从 Cursor 中获取数据的 MIME 类型列的值，并赋值给成员变量 mDataMimeType
        mDataMimeType = c.getString(DATA_MIME_TYPE_COLUMN);
        // 获取数据内容 从 Cursor 中获取数据的 CONTENT 列的值，并赋值给成员变量 mDataContent
        mDataContent = c.getString(DATA_CONTENT_COLUMN);
        // 获取数据的其他信息 从 Cursor 中获取数据的 DATA1 列的值，并赋值给成员变量 mDataContentData1
        mDataContentData1 = c.getLong(DATA_CONTENT_DATA_1_COLUMN);
        // 获取数据的其他信息 从  Cursor 中获取数据的 DATA3 列的值，并赋值给成员变量 mDataContentData3
        mDataContentData3 = c.getString(DATA_CONTENT_DATA_3_COLUMN);
    }

    // 设置数据
    //根据传入的 JSON 对象设置 SqlData 实例的内容，并记录数据的差异。
    //该方法会从 JSON 对象中提取数据的各个字段（如 ID、MIME 类型、内容等），
    // 并将其赋值给当前实例的成员变量。如果数据正在创建或者与当前值不同，则将这些差异记录到 mDiffDataValues 中，以便后续提交到数据库。
    public void setContent(JSONObject js) throws JSONException {
        // 获取数据ID 从 JSON 对象中获取数据的 ID 列的值，并赋值给变量 dataId
        long dataId = js.has(DataColumns.ID) ? js.getLong(DataColumns.ID) : INVALID_ID;
        // 如果正在创建数据或者数据ID不匹配，则将数据ID添加到 mDiffDataValues 中
        if (mIsCreate || mDataId != dataId) {
            mDiffDataValues.put(DataColumns.ID, dataId);
        }
        // 更新当前实例数据ID 将变量 dataId 的值赋值给成员变量 mDataId
        mDataId = dataId;
        // 获取数据类型 从 JSON 对象中获取数据的 MIME 类型列的值，并赋值给变量 dataMimeType
        String dataMimeType = js.has(DataColumns.MIME_TYPE) ? js.getString(DataColumns.MIME_TYPE)
                : DataConstants.NOTE;
        // 如果正在创建数据或者数据类型不匹配，则将数据类型添加到 mDiffDataValues 中
        if (mIsCreate || !mDataMimeType.equals(dataMimeType)) {
            mDiffDataValues.put(DataColumns.MIME_TYPE, dataMimeType);
        }
        // 更新当前实例数据类型 将变量 dataMimeType 的值赋值给成员变量 mDataMimeType
        mDataMimeType = dataMimeType;
        // 获取数据内容 从 JSON 对象中获取数据的 CONTENT 列的值，并赋值给变量 dataContent
        String dataContent = js.has(DataColumns.CONTENT) ? js.getString(DataColumns.CONTENT) : "";
        // 如果正在创建数据或者数据内容不匹配，则将数据内容添加到 mDiffDataValues 中
        if (mIsCreate || !mDataContent.equals(dataContent)) {
            mDiffDataValues.put(DataColumns.CONTENT, dataContent);
        }
        // 更新数据内容 将变量 dataContent 的值赋值给成员变量 mDataContent
        mDataContent = dataContent;
        // 获取数据的其他信息 从 JSON 对象中获取数据的 DATA1 列的值，并赋值给变量 dataContentData1
        long dataContentData1 = js.has(DataColumns.DATA1) ? js.getLong(DataColumns.DATA1) : 0;
        // 如果正在创建数据或者数据的其他信息不匹配，则将数据的其他信息添加到 mDiffDataValues 中
        if (mIsCreate || mDataContentData1 != dataContentData1) {
            mDiffDataValues.put(DataColumns.DATA1, dataContentData1);
        }
        // 更新当前实例数据的其他信息 将变量 dataContentData1 的值赋值给成员变量 mDataContentData1  
        mDataContentData1 = dataContentData1;
        // 获取数据的其他信息 从 JSON 对象中获取数据的 DATA3 列的值，并赋值给变量 dataContentData3
        String dataContentData3 = js.has(DataColumns.DATA3) ? js.getString(DataColumns.DATA3) : "";
        // 如果正在创建数据或者数据的其他信息不匹配，则将数据的其他信息添加到 mDiffDataValues 中
        if (mIsCreate || !mDataContentData3.equals(dataContentData3)) {
            mDiffDataValues.put(DataColumns.DATA3, dataContentData3);
        }
        // 更新当前实例数据的其他信息 将变量 dataContentData3 的值赋值给成员变量 mDataContentData3  
        mDataContentData3 = dataContentData3;
    }

    // 获取内容
    public JSONObject getContent() throws JSONException {
        // 如果mIsCreate为true，说明还没有在数据库中创建
        if (mIsCreate) {
            Log.e(TAG, "it seems that we haven't created this in database yet");
            return null;
        }
        // 创建一个JSONObject对象并填充数据
        JSONObject js = new JSONObject();
        // 将mDataId放入JSONObject中
        js.put(DataColumns.ID, mDataId);
        // 将mDataMimeType放入JSONObject中
        js.put(DataColumns.MIME_TYPE, mDataMimeType);
        // 将mDataContent放入JSONObject中
        js.put(DataColumns.CONTENT, mDataContent);
        // 将mDataContentData1放入JSONObject中
        js.put(DataColumns.DATA1, mDataContentData1);
        // 将mDataContentData3放入JSONObject中
        js.put(DataColumns.DATA3, mDataContentData3);
        // 返回JSONObject对象
        return js;
    }

    // 提交数据
    public void commit(long noteId, boolean validateVersion, long version) {

        // 如果是创建
        if (mIsCreate) {
            // 如果数据ID无效且数据值中包含ID列
            if (mDataId == INVALID_ID && mDiffDataValues.containsKey(DataColumns.ID)) {
                // 移除ID列
                mDiffDataValues.remove(DataColumns.ID);
            }

            // 添加NOTE_ID列
            mDiffDataValues.put(DataColumns.NOTE_ID, noteId);
            // 插入数据
            Uri uri = mContentResolver.insert(Notes.CONTENT_DATA_URI, mDiffDataValues);
            try {
                // 获取数据ID
                mDataId = Long.valueOf(uri.getPathSegments().get(1));
            } catch (NumberFormatException e) {
                // 打印错误日志
                Log.e(TAG, "Get note id error :" + e.toString());
                // 抛出异常
                throw new ActionFailureException("create note failed");
            }
        } else {
            // 如果数据值不为空
            if (mDiffDataValues.size() > 0) {
                // 更新结果
                int result = 0;
                // 如果不验证版本
                if (!validateVersion) {
                    // 更新数据
                    result = mContentResolver.update(ContentUris.withAppendedId(
                            Notes.CONTENT_DATA_URI, mDataId), mDiffDataValues, null, null);
                } else {
                    // 更新数据，验证版本
                    result = mContentResolver.update(ContentUris.withAppendedId(
                            Notes.CONTENT_DATA_URI, mDataId), mDiffDataValues,
                            " ? in (SELECT " + NoteColumns.ID + " FROM " + TABLE.NOTE
                                    + " WHERE " + NoteColumns.VERSION + "=?)", new String[] {
                                    String.valueOf(noteId), String.valueOf(version)
                            });
                }
                // 如果更新结果为0
                if (result == 0) {
                    // 打印警告日志
                    Log.w(TAG, "there is no update. maybe user updates note when syncing");
                }
            }
        }

        // 清空数据值
        mDiffDataValues.clear();
        // 设置为非创建
        mIsCreate = false;
    }

    // 获取数据ID
    public long getId() {
        return mDataId;
    }
}
