//开源许可协议声明
/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
//类说明
package net.micode.notes.gtask.data;
//导入Android系统提供的Cursor类，用于数据库查询操作。Cursor类表示数据库查询结果集，可以用来遍历查询结果。       
import android.database.Cursor;
import android.text.TextUtils;
import android.util.Log;
            
import net.micode.notes.data.Notes;
import net.micode.notes.data.Notes.DataColumns;
import net.micode.notes.data.Notes.DataConstants;
import net.micode.notes.data.Notes.NoteColumns;
import net.micode.notes.gtask.exception.ActionFailureException;
import net.micode.notes.tool.GTaskStringUtils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;


public class Task extends Node {
    // 定义一个常量，用于打印日志
    private static final String TAG = Task.class.getSimpleName();

    // 定义一个布尔变量，表示任务是否完成
    private boolean mCompleted;

    // 定义一个字符串变量，用于存储任务的备注
    private String mNotes;

    // 定义一个JSONObject变量，用于存储任务的元信息
    private JSONObject mMetaInfo;

    // 定义一个Task变量，用于存储任务的前一个兄弟任务
    private Task mPriorSibling;

    // 定义一个TaskList变量，用于存储任务的父任务列表
    private TaskList mParent;

    // 构造函数，初始化任务
    public Task() {
        super();
        mCompleted = false;
        mNotes = null;
        mPriorSibling = null;
        mParent = null;
        mMetaInfo = null;
    }

    // 获取创建任务的JSON对象
    public JSONObject getCreateAction(int actionId) {
        // 创建一个JSONObject对象
        JSONObject js = new JSONObject();

        try {
            // action_type
            // 将action_type设置为create
            js.put(GTaskStringUtils.GTASK_JSON_ACTION_TYPE,
                    GTaskStringUtils.GTASK_JSON_ACTION_TYPE_CREATE);

            // action_id
            // 将action_id设置为传入的参数
            js.put(GTaskStringUtils.GTASK_JSON_ACTION_ID, actionId);

            // index
            // 将index设置为父任务的子任务索引
            js.put(GTaskStringUtils.GTASK_JSON_INDEX, mParent.getChildTaskIndex(this));

            // entity_delta
            // 创建一个JSONObject对象，用于存储任务信息
            JSONObject entity = new JSONObject();
            // 将任务名称放入entity中
            entity.put(GTaskStringUtils.GTASK_JSON_NAME, getName());
            // 将创建者ID放入entity中
            entity.put(GTaskStringUtils.GTASK_JSON_CREATOR_ID, "null");
            // 将任务类型放入entity中
            entity.put(GTaskStringUtils.GTASK_JSON_ENTITY_TYPE,
                    GTaskStringUtils.GTASK_JSON_TYPE_TASK);
            // 如果任务有备注，则将备注放入entity中
            if (getNotes() != null) {
                entity.put(GTaskStringUtils.GTASK_JSON_NOTES, getNotes());
            }
            // 将entity放入js中
            js.put(GTaskStringUtils.GTASK_JSON_ENTITY_DELTA, entity);

            // parent_id
            // 将父任务的GID放入js中
            js.put(GTaskStringUtils.GTASK_JSON_PARENT_ID, mParent.getGid());

            // dest_parent_type
            // 将目标父任务类型设置为group
            js.put(GTaskStringUtils.GTASK_JSON_DEST_PARENT_TYPE,
                    GTaskStringUtils.GTASK_JSON_TYPE_GROUP);

            // list_id
            // 将父任务的GID放入js中
            js.put(GTaskStringUtils.GTASK_JSON_LIST_ID, mParent.getGid());

            // prior_sibling_id
            // 如果有前一个兄弟任务，则将前一个兄弟任务的GID放入js中
            if (mPriorSibling != null) {
                js.put(GTaskStringUtils.GTASK_JSON_PRIOR_SIBLING_ID, mPriorSibling.getGid());
            }

        } catch (JSONException e) {
            // 如果发生JSONException，则打印错误日志，并抛出ActionFailureException异常
            Log.e(TAG, e.toString());
            e.printStackTrace();
            throw new ActionFailureException("fail to generate task-create jsonobject");
        }

        // 返回JSONObject对象
        return js;
    }

    // 获取更新任务的JSON对象
    public JSONObject getUpdateAction(int actionId) {
        // 创建一个JSONObject对象
        JSONObject js = new JSONObject();

        try {
            // action_type
            
            js.put(GTaskStringUtils.GTASK_JSON_ACTION_TYPE,
                    GTaskStringUtils.GTASK_JSON_ACTION_TYPE_UPDATE);

            // action_id
            js.put(GTaskStringUtils.GTASK_JSON_ACTION_ID, actionId);

            // id
            js.put(GTaskStringUtils.GTASK_JSON_ID, getGid());

            // entity_delta
            JSONObject entity = new JSONObject();
            entity.put(GTaskStringUtils.GTASK_JSON_NAME, getName());
            if (getNotes() != null) {
                entity.put(GTaskStringUtils.GTASK_JSON_NOTES, getNotes());
            }
            entity.put(GTaskStringUtils.GTASK_JSON_DELETED, getDeleted());
            js.put(GTaskStringUtils.GTASK_JSON_ENTITY_DELTA, entity);

        } catch (JSONException e) {
            Log.e(TAG, e.toString());
            e.printStackTrace();
            throw new ActionFailureException("fail to generate task-update jsonobject");
        }

        return js;
    }

    // 根据远程JSON设置任务内容
    public void setContentByRemoteJSON(JSONObject js) {
        // 根据远程JSON设置内容
        if (js != null) {
            try {
                // id
                if (js.has(GTaskStringUtils.GTASK_JSON_ID)) {
                    // 设置id
                    setGid(js.getString(GTaskStringUtils.GTASK_JSON_ID));
                }

                // last_modified
                if (js.has(GTaskStringUtils.GTASK_JSON_LAST_MODIFIED)) {
                    // 设置最后修改时间
                    setLastModified(js.getLong(GTaskStringUtils.GTASK_JSON_LAST_MODIFIED));
                }

                // name
                if (js.has(GTaskStringUtils.GTASK_JSON_NAME)) {
                    // 设置名称
                    setName(js.getString(GTaskStringUtils.GTASK_JSON_NAME));
                }

                // notes
                if (js.has(GTaskStringUtils.GTASK_JSON_NOTES)) {
                    // 设置备注
                    setNotes(js.getString(GTaskStringUtils.GTASK_JSON_NOTES));
                }

                // deleted
                if (js.has(GTaskStringUtils.GTASK_JSON_DELETED)) {
                    // 设置是否删除
                    setDeleted(js.getBoolean(GTaskStringUtils.GTASK_JSON_DELETED));
                }

                // completed
                if (js.has(GTaskStringUtils.GTASK_JSON_COMPLETED)) {
                    // 设置是否完成
                    setCompleted(js.getBoolean(GTaskStringUtils.GTASK_JSON_COMPLETED));
                }
            } catch (JSONException e) {
                Log.e(TAG, e.toString());
                e.printStackTrace();
                // 抛出异常
                throw new ActionFailureException("fail to get task content from jsonobject");
            }
        }
    }

    // 根据本地JSON设置任务内容
    public void setContentByLocalJSON(JSONObject js) {
        // 如果JSON为空或者没有META_HEAD_NOTE或META_HEAD_DATA字段，则打印警告日志
        if (js == null || !js.has(GTaskStringUtils.META_HEAD_NOTE)
                || !js.has(GTaskStringUtils.META_HEAD_DATA)) {
            Log.w(TAG, "setContentByLocalJSON: nothing is avaiable");
        }

        try {
            // 获取META_HEAD_NOTE字段对应的JSONObject
            JSONObject note = js.getJSONObject(GTaskStringUtils.META_HEAD_NOTE);
            JSONArray dataArray = js.getJSONArray(GTaskStringUtils.META_HEAD_DATA);

            // 如果note的类型不是Notes.TYPE_NOTE，则打印错误日志并返回
            if (note.getInt(NoteColumns.TYPE) != Notes.TYPE_NOTE) {
                Log.e(TAG, "invalid type");
                return;
            }

            // 遍历dataArray中的每个JSONObject
            for (int i = 0; i < dataArray.length(); i++) {
                JSONObject data = dataArray.getJSONObject(i);
                // 如果data的MIME_TYPE字段为DataConstants.NOTE，则设置name字段为data的CONTENT字段
                if (TextUtils.equals(data.getString(DataColumns.MIME_TYPE), DataConstants.NOTE)) {
                    setName(data.getString(DataColumns.CONTENT));
                    break;
                }
            }

        } catch (JSONException e) {
            Log.e(TAG, e.toString());
            e.printStackTrace();
        }
    }

    // 获取本地JSON对象
    public JSONObject getLocalJSONFromContent() {
        // 获取名称
        String name = getName();
        try {
            // 如果mMetaInfo为空，说明是新建的任务
            if (mMetaInfo == null) {
                // new task created from web
                // 如果名称为空，说明是空的笔记
                if (name == null) {
                    Log.w(TAG, "the note seems to be an empty one");
                    return null;
                }

                // 创建一个新的JSONObject
                JSONObject js = new JSONObject();
                // 创建一个新的JSONObject，用于存储笔记信息
                JSONObject note = new JSONObject();
                // 创建一个新的JSONArray，用于存储数据
                JSONArray dataArray = new JSONArray();
                // 创建一个新的JSONObject，用于存储数据信息
                JSONObject data = new JSONObject();
                // 将名称存入数据信息
                data.put(DataColumns.CONTENT, name);
                // 将数据信息存入数据数组
                dataArray.put(data);
                // 将数据数组存入笔记信息
                js.put(GTaskStringUtils.META_HEAD_DATA, dataArray);
                // 将笔记信息存入JSONObject
                note.put(NoteColumns.TYPE, Notes.TYPE_NOTE);
                js.put(GTaskStringUtils.META_HEAD_NOTE, note);
                return js;
            } else {
                // synced task

                JSONObject note = mMetaInfo.getJSONObject(GTaskStringUtils.META_HEAD_NOTE);

                JSONArray dataArray = mMetaInfo.getJSONArray(GTaskStringUtils.META_HEAD_DATA);

                for (int i = 0; i < dataArray.length(); i++) {

                    JSONObject data = dataArray.getJSONObject(i);
                    if (TextUtils.equals(data.getString(DataColumns.MIME_TYPE), DataConstants.NOTE)) {
                        data.put(DataColumns.CONTENT, getName());
                        break;
                    }
                }
                // 将数据信息存入数据数组
                note.put(NoteColumns.TYPE, Notes.TYPE_NOTE);
                return mMetaInfo;
            }

        } catch (JSONException e) {
            Log.e(TAG, e.toString());
            e.printStackTrace();
            return null;
        }
    }

    // 设置任务的元信息
    // 设置元数据信息
    public void setMetaInfo(MetaData metaData) {
        // 如果元数据不为空且元数据的注释不为空
        if (metaData != null && metaData.getNotes() != null) {
            try {
                // 将元数据的注释转换为JSON对象
                mMetaInfo = new JSONObject(metaData.getNotes());
            } catch (JSONException e) {
                // 如果转换失败，打印警告日志
                Log.w(TAG, e.toString());
                // 将元数据信息设置为空
                mMetaInfo = null;
            }
        }
    }

    // 获取同步动作
    public int getSyncAction(Cursor c) {
        try {
            // 获取noteInfo
            JSONObject noteInfo = null;
            if (mMetaInfo != null && mMetaInfo.has(GTaskStringUtils.META_HEAD_NOTE)) {
                noteInfo = mMetaInfo.getJSONObject(GTaskStringUtils.META_HEAD_NOTE);
            }

            // 如果noteInfo为空，则返回SYNC_ACTION_UPDATE_REMOTE
            if (noteInfo == null) {
                Log.w(TAG, "it seems that note meta has been deleted");
                return SYNC_ACTION_UPDATE_REMOTE;
            }

            // 如果noteInfo中没有ID，则返回SYNC_ACTION_UPDATE_LOCAL
            if (!noteInfo.has(NoteColumns.ID)) {
                Log.w(TAG, "remote note id seems to be deleted");
                return SYNC_ACTION_UPDATE_LOCAL;
            }

            // validate the note id now
            // 验证note id是否一致
            if (c.getLong(SqlNote.ID_COLUMN) != noteInfo.getLong(NoteColumns.ID)) {
                Log.w(TAG, "note id doesn't match");
                return SYNC_ACTION_UPDATE_LOCAL;
            }
             // 检查本地是否有修改                                  
            if (c.getInt(SqlNote.LOCAL_MODIFIED_COLUMN) == 0) {
                // there is no local update本地没有更新，继续比较本地同步 ID 和远程任务的最后修改时间
                if (c.getLong(SqlNote.SYNC_ID_COLUMN) == getLastModified()) {
                    // no update both side本地和远程都没有更新，无需同步操作
                    return SYNC_ACTION_NONE;
                } else {
                    // apply remote to local远程有更新，本地无更新，需要将远程的更新应用到本地
                    return SYNC_ACTION_UPDATE_LOCAL;
                }
            } else {
                // validate gtask id 本地有更新，验证本地存储的 Google 任务 ID 和当前任务的全局唯一标识符是否一致
                if (!c.getString(SqlNote.GTASK_ID_COLUMN).equals(getGid())) {
                    // 两者不一致，记录错误日志并返回错误同步动作
                    Log.e(TAG, "gtask id doesn't match");
                    return SYNC_ACTION_ERROR;
                }
                if (c.getLong(SqlNote.SYNC_ID_COLUMN) == getLastModified()) {
                    // local modification only  本地有修改，远程无更新，需要将本地的修改同步到远程
                    return SYNC_ACTION_UPDATE_REMOTE;
                } else {
                     // 本地和远程都有更新，发生同步冲突
                    return SYNC_ACTION_UPDATE_CONFLICT;
                }
            }
        } catch (Exception e) {
            // 捕获异常，记录错误日志并打印堆栈信息
            Log.e(TAG, e.toString());
            e.printStackTrace();
        }
        // 发生异常或其他未知情况，返回错误同步动作
        return SYNC_ACTION_ERROR;
    }

    // 判断任务是否值得保存
    public boolean isWorthSaving() {
        // 如果mMetaInfo不为空，或者getName()返回的字符串不为空且长度大于0，或者getNotes()返回的字符串不为空且长度大于0，则返回true
        return mMetaInfo != null || (getName() != null && getName().trim().length() > 0)
                || (getNotes() != null && getNotes().trim().length() > 0);
    }

    // 设置任务是否完成
    public void setCompleted(boolean completed) {
        // 将传入的参数completed赋值给成员变量mCompleted
        this.mCompleted = completed;
    }

    // 设置任务的备注
    public void setNotes(String notes) {
        // 将传入的备注赋值给成员变量mNotes
        this.mNotes = notes;
    }

    // 设置任务的前一个兄弟任务
    public void setPriorSibling(Task priorSibling) {
        // 将传入的前一个兄弟任务赋值给成员变量mPriorSibling
        this.mPriorSibling = priorSibling;
    }

    // 设置任务的父任务列表
    public void setParent(TaskList parent) {
        // 将传入的父任务列表赋值给成员变量mParent
        this.mParent = parent;
    }

    // 获取任务是否完成
    public boolean getCompleted() {
        // 返回任务是否完成
        return this.mCompleted;
    }

    // 获取任务的备注
    public String getNotes() {
        // 返回备注
        return this.mNotes;
    }

    // 获取任务的前一个兄弟任务
    public Task getPriorSibling() {
        return this.mPriorSibling;
    }

    // 获取任务的父任务列表
    public TaskList getParent() {
        // 返回父任务列表
        return this.mParent;
    }

}
